@@ -18,6 +18,7 @@
 import sys
 import warnings
 from typing import Any, Dict, List, Optional, TYPE_CHECKING
+import functools
 
 import numpy as np
 
@@ -44,12 +45,15 @@ from pyspark.ml.util import (
     JavaMLReadable,
     GeneralJavaMLWritable,
     HasTrainingSummary,
+    try_remote_attribute_relation,
+    invoke_helper_relation,
 )
 from pyspark.ml.wrapper import JavaEstimator, JavaModel, JavaParams, JavaWrapper
-from pyspark.ml.common import inherit_doc, _java2py
+from pyspark.ml.common import inherit_doc
 from pyspark.ml.stat import MultivariateGaussian
 from pyspark.sql import DataFrame
 from pyspark.ml.linalg import Vector, Matrix
+from pyspark.sql.utils import is_remote
 
 if TYPE_CHECKING:
     from pyspark.ml._typing import M
@@ -91,6 +95,7 @@ class ClusteringSummary(JavaWrapper):
 
     @property
     @since("2.1.0")
+    @try_remote_attribute_relation
     def predictions(self) -> DataFrame:
         """
         DataFrame produced by the model's `transform` method.
@@ -115,6 +120,7 @@ class ClusteringSummary(JavaWrapper):
 
     @property
     @since("2.1.0")
+    @try_remote_attribute_relation
     def cluster(self) -> DataFrame:
         """
         DataFrame of predicted cluster centers for each training data point.
@@ -208,6 +214,14 @@ class GaussianMixtureModel(
         """
         return self._set(probabilityCol=value)
 
+    @property
+    @since("4.1.0")
+    def numFeatures(self) -> int:
+        """
+        Number of features, i.e., length of Vectors which this transforms.
+        """
+        return self._call_java("numFeatures")
+
     @property
     @since("2.0.0")
     def weights(self) -> List[float]:
@@ -225,19 +239,14 @@ class GaussianMixtureModel(
         Array of :py:class:`MultivariateGaussian` where gaussians[i] represents
         the Multivariate Gaussian (Normal) Distribution for Gaussian i
         """
-        from pyspark.core.context import SparkContext
-
-        sc = SparkContext._active_spark_context
-        assert sc is not None and self._java_obj is not None
-
-        jgaussians = self._java_obj.gaussians()
         return [
-            MultivariateGaussian(_java2py(sc, jgaussian.mean()), _java2py(sc, jgaussian.cov()))
-            for jgaussian in jgaussians
+            MultivariateGaussian(row.mean.asML(), row.cov.asML())
+            for row in self.gaussiansDF.collect()
         ]
 
     @property
     @since("2.0.0")
+    @try_remote_attribute_relation
     def gaussiansDF(self) -> DataFrame:
         """
         Retrieve Gaussian distributions as a DataFrame.
@@ -246,20 +255,6 @@ class GaussianMixtureModel(
         """
         return self._call_java("gaussiansDF")
 
-    @property
-    @since("2.1.0")
-    def summary(self) -> "GaussianMixtureSummary":
-        """
-        Gets summary (cluster assignments, cluster sizes) of the model trained on the
-        training set. An exception is thrown if no summary exists.
-        """
-        if self.hasSummary:
-            return GaussianMixtureSummary(super(GaussianMixtureModel, self).summary)
-        else:
-            raise RuntimeError(
-                "No training summary available for this %s" % self.__class__.__name__
-            )
-
     @since("3.0.0")
     def predict(self, value: Vector) -> int:
         """
@@ -274,6 +269,10 @@ class GaussianMixtureModel(
         """
         return self._call_java("predictProbability", value)
 
+    @property
+    def _summaryCls(self) -> type:
+        return GaussianMixtureSummary
+
 
 @inherit_doc
 class GaussianMixture(
@@ -349,15 +348,15 @@ class GaussianMixture(
     >>> gaussians[0].mean
     DenseVector([0.825, 0.8675])
     >>> gaussians[0].cov
-    DenseMatrix(2, 2, [0.0056, -0.0051, -0.0051, 0.0046], 0)
+    DenseMatrix(2, 2, [0.0056, -0.0051, -0.0051, 0.0046], False)
     >>> gaussians[1].mean
     DenseVector([-0.87, -0.72])
     >>> gaussians[1].cov
-    DenseMatrix(2, 2, [0.0016, 0.0016, 0.0016, 0.0016], 0)
+    DenseMatrix(2, 2, [0.0016, 0.0016, 0.0016, 0.0016], False)
     >>> gaussians[2].mean
     DenseVector([-0.055, -0.075])
     >>> gaussians[2].cov
-    DenseMatrix(2, 2, [0.002, -0.0011, -0.0011, 0.0006], 0)
+    DenseMatrix(2, 2, [0.002, -0.0011, -0.0011, 0.0006], False)
     >>> model.gaussiansDF.select("mean").head()
     Row(mean=DenseVector([0.825, 0.8675]))
     >>> model.gaussiansDF.select("cov").head()
@@ -539,6 +538,7 @@ class GaussianMixtureSummary(ClusteringSummary):
 
     @property
     @since("2.1.0")
+    @try_remote_attribute_relation
     def probability(self) -> DataFrame:
         """
         DataFrame of probabilities of each cluster for each training data point.
@@ -681,21 +681,16 @@ class KMeansModel(
     @since("1.5.0")
     def clusterCenters(self) -> List[np.ndarray]:
         """Get the cluster centers, represented as a list of NumPy arrays."""
-        return [c.toArray() for c in self._call_java("clusterCenters")]
+        matrix = self._call_java("clusterCenterMatrix")
+        return [vec for vec in matrix.toArray()]
 
     @property
-    @since("2.1.0")
-    def summary(self) -> KMeansSummary:
+    @since("4.1.0")
+    def numFeatures(self) -> int:
         """
-        Gets summary (cluster assignments, cluster sizes) of the model trained on the
-        training set. An exception is thrown if no summary exists.
+        Number of features, i.e., length of Vectors which this transforms.
         """
-        if self.hasSummary:
-            return KMeansSummary(super(KMeansModel, self).summary)
-        else:
-            raise RuntimeError(
-                "No training summary available for this %s" % self.__class__.__name__
-            )
+        return self._call_java("numFeatures")
 
     @since("3.0.0")
     def predict(self, value: Vector) -> int:
@@ -704,6 +699,10 @@ class KMeansModel(
         """
         return self._call_java("predict", value)
 
+    @property
+    def _summaryCls(self) -> type:
+        return KMeansSummary
+
 
 @inherit_doc
 class KMeans(JavaEstimator[KMeansModel], _KMeansParams, JavaMLWritable, JavaMLReadable["KMeans"]):
@@ -1001,7 +1000,8 @@ class BisectingKMeansModel(
     @since("2.0.0")
     def clusterCenters(self) -> List[np.ndarray]:
         """Get the cluster centers, represented as a list of NumPy arrays."""
-        return [c.toArray() for c in self._call_java("clusterCenters")]
+        matrix = self._call_java("clusterCenterMatrix")
+        return [vec for vec in matrix.toArray()]
 
     @since("2.0.0")
     def computeCost(self, dataset: DataFrame) -> float:
@@ -1022,18 +1022,12 @@ class BisectingKMeansModel(
         return self._call_java("computeCost", dataset)
 
     @property
-    @since("2.1.0")
-    def summary(self) -> "BisectingKMeansSummary":
+    @since("4.1.0")
+    def numFeatures(self) -> int:
         """
-        Gets summary (cluster assignments, cluster sizes) of the model trained on the
-        training set. An exception is thrown if no summary exists.
+        Number of features, i.e., length of Vectors which this transforms.
         """
-        if self.hasSummary:
-            return BisectingKMeansSummary(super(BisectingKMeansModel, self).summary)
-        else:
-            raise RuntimeError(
-                "No training summary available for this %s" % self.__class__.__name__
-            )
+        return self._call_java("numFeatures")
 
     @since("3.0.0")
     def predict(self, value: Vector) -> int:
@@ -1042,6 +1036,10 @@ class BisectingKMeansModel(
         """
         return self._call_java("predict", value)
 
+    @property
+    def _summaryCls(self) -> type:
+        return BisectingKMeansSummary
+
 
 @inherit_doc
 class BisectingKMeans(
@@ -1506,6 +1504,7 @@ class LDAModel(JavaModel, _LDAParams):
         return self._call_java("logPerplexity", dataset)
 
     @since("2.0.0")
+    @try_remote_attribute_relation
     def describeTopics(self, maxTermsPerTopic: int = 10) -> DataFrame:
         """
         Return the topics described by their top-weighted terms.
@@ -1534,6 +1533,7 @@ class DistributedLDAModel(LDAModel, JavaMLReadable["DistributedLDAModel"], JavaM
     .. versionadded:: 2.0.0
     """
 
+    @functools.cache
     @since("2.0.0")
     def toLocal(self) -> "LocalLDAModel":
         """
@@ -1543,6 +1543,8 @@ class DistributedLDAModel(LDAModel, JavaMLReadable["DistributedLDAModel"], JavaM
         .. warning:: This involves collecting a large :py:func:`topicsMatrix` to the driver.
         """
         model = LocalLDAModel(self._call_java("toLocal"))
+        if is_remote():
+            return model
 
         # SPARK-10931: Temporary fix to be removed once LDAModel defines Params
         model._create_params_from_java()
@@ -2145,9 +2147,22 @@ class PowerIterationClustering(
             - id: Long
             - cluster: Int
         """
-        self._transfer_params_to_java()
         assert self._java_obj is not None
 
+        if is_remote():
+            return invoke_helper_relation(
+                "powerIterationClusteringAssignClusters",
+                dataset,
+                self.getK(),
+                self.getMaxIter(),
+                self.getInitMode(),
+                self.getSrcCol(),
+                self.getDstCol(),
+                self.getWeightCol() if self.isDefined(self.weightCol) else "",
+            )
+
+        self._transfer_params_to_java()
+
         jdf = self._java_obj.assignClusters(dataset._jdf)
         return DataFrame(jdf, dataset.sparkSession)
 
