@@ -18,14 +18,13 @@
 package org.apache.spark.sql.streaming
 
 import java.io.File
-import java.nio.charset.StandardCharsets.UTF_8
+import java.nio.file.Files
 import java.util.Collections
 import java.util.concurrent.CountDownLatch
 
 import scala.collection.mutable
 import scala.util.{Success, Try}
 
-import org.apache.commons.io.FileUtils
 import org.apache.commons.lang3.RandomStringUtils
 import org.apache.hadoop.fs.Path
 import org.mockito.Mockito.when
@@ -36,21 +35,26 @@ import org.scalatestplus.mockito.MockitoSugar
 
 import org.apache.spark.{SparkException, SparkUnsupportedOperationException, TestUtils}
 import org.apache.spark.internal.Logging
-import org.apache.spark.sql.{AnalysisException, DataFrame, Dataset, Row, SaveMode}
+import org.apache.spark.sql.{AnalysisException, Row, SaveMode}
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.plans.logical.{CTERelationDef, CTERelationRef, LocalRelation}
 import org.apache.spark.sql.catalyst.streaming.InternalOutputModes.Complete
 import org.apache.spark.sql.catalyst.types.DataTypeUtils.toAttributes
+import org.apache.spark.sql.classic.{DataFrame, Dataset}
 import org.apache.spark.sql.connector.read.InputPartition
 import org.apache.spark.sql.connector.read.streaming.{Offset => OffsetV2, ReadLimit}
-import org.apache.spark.sql.execution.exchange.ReusedExchangeExec
+import org.apache.spark.sql.execution.exchange.{REQUIRED_BY_STATEFUL_OPERATOR, ReusedExchangeExec, ShuffleExchangeExec}
 import org.apache.spark.sql.execution.streaming._
+import org.apache.spark.sql.execution.streaming.checkpointing.{CheckpointFileManager, OffsetSeqMetadata}
+import org.apache.spark.sql.execution.streaming.runtime.{LongOffset, MemoryStream, MetricsReporter, StreamExecution, StreamingExecutionRelation, StreamingQueryWrapper}
 import org.apache.spark.sql.execution.streaming.sources.{MemorySink, TestForeachWriter}
+import org.apache.spark.sql.execution.streaming.state.{HDFSBackedStateStoreProvider, RocksDBStateStoreProvider, StateStoreCheckpointLocationNotEmpty}
 import org.apache.spark.sql.functions._
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.streaming.util.{BlockingSource, MockSourceProvider, StreamManualClock}
 import org.apache.spark.sql.types.StructType
 import org.apache.spark.tags.SlowSQLTest
+import org.apache.spark.util.Utils
 
 @SlowSQLTest
 class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging with MockitoSugar {
@@ -103,12 +107,14 @@ class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging wi
       var cpDir: String = null
 
       def startQuery(restart: Boolean): StreamingQuery = {
-        if (cpDir == null || !restart) cpDir = s"$dir/${RandomStringUtils.randomAlphabetic(10)}"
+        if (cpDir == null || !restart) {
+          cpDir = s"$dir/${RandomStringUtils.secure.nextAlphabetic(10)}"
+        }
         MemoryStream[Int].toDS().groupBy().count()
           .writeStream
           .format("memory")
           .outputMode("complete")
-          .queryName(s"name${RandomStringUtils.randomAlphabetic(10)}")
+          .queryName(s"name${RandomStringUtils.secure.nextAlphabetic(10)}")
           .option("checkpointLocation", cpDir)
           .start()
       }
@@ -1094,7 +1100,7 @@ class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging wi
     val inputDir = new File(input.toURI)
 
     // Copy test files to tempDir so that we won't modify the original data.
-    FileUtils.copyDirectory(inputDir, dir)
+    Utils.copyDirectory(inputDir, dir)
 
     // Spark 2.4 and earlier escaped the _spark_metadata path once
     val legacySparkMetadataDir = new File(
@@ -1105,10 +1111,10 @@ class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging wi
     // Ideally we should copy "_spark_metadata" directly like what the user is supposed to do to
     // migrate to new version. However, in our test, "tempDir" will be different in each run and
     // we need to fix the absolute path in the metadata to match "tempDir".
-    val sparkMetadata = FileUtils.readFileToString(new File(legacySparkMetadataDir, "0"), UTF_8)
-    FileUtils.write(
-      new File(legacySparkMetadataDir, "0"),
-      sparkMetadata.replaceAll("TEMPDIR", dir.getCanonicalPath), UTF_8)
+    val sparkMetadata = Files.readString(new File(legacySparkMetadataDir, "0").toPath)
+    Files.writeString(
+      new File(legacySparkMetadataDir, "0").toPath,
+      sparkMetadata.replaceAll("TEMPDIR", dir.getCanonicalPath))
   }
 
   test("detect escaped path and report the migration guide") {
@@ -1159,7 +1165,7 @@ class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging wi
       assertMigrationError(e2.getMessage, sparkMetadataDir, legacySparkMetadataDir)
 
       // Move "_spark_metadata" to fix the file sink and test the checkpoint path.
-      FileUtils.moveDirectory(legacySparkMetadataDir, sparkMetadataDir)
+      Utils.moveDirectory(legacySparkMetadataDir, sparkMetadataDir)
 
       // Restarting the streaming query should detect the legacy
       // checkpoint path and throw an error.
@@ -1173,7 +1179,7 @@ class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging wi
       assertMigrationError(e3.getMessage, checkpointDir, legacyCheckpointDir)
 
       // Fix the checkpoint path and verify that the user can migrate the issue by moving files.
-      FileUtils.moveDirectory(legacyCheckpointDir, checkpointDir)
+      Utils.moveDirectory(legacyCheckpointDir, checkpointDir)
 
       val q = inputData.toDF()
         .writeStream
@@ -1448,6 +1454,108 @@ class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging wi
     }
   }
 
+  test("SPARK-49905 shuffle added by stateful operator should use the shuffle origin " +
+    "`REQUIRED_BY_STATEFUL_OPERATOR`") {
+    val inputData = MemoryStream[Int]
+
+    // Use the streaming aggregation as an example - all stateful operators are using the same
+    // distribution, named `StatefulOpClusteredDistribution`.
+    val df = inputData.toDF().groupBy("value").count()
+
+    testStream(df, OutputMode.Update())(
+      AddData(inputData, 1, 2, 3, 1, 2, 3),
+      CheckAnswer((1, 2), (2, 2), (3, 2)),
+      Execute { qe =>
+        val shuffleOpt = qe.lastExecution.executedPlan.collect {
+          case s: ShuffleExchangeExec => s
+        }
+
+        assert(shuffleOpt.nonEmpty, "No shuffle exchange found in the query plan")
+        assert(shuffleOpt.head.shuffleOrigin === REQUIRED_BY_STATEFUL_OPERATOR)
+      }
+    )
+  }
+
+  private val TEST_PROVIDERS = Seq(
+    classOf[HDFSBackedStateStoreProvider].getName,
+    classOf[RocksDBStateStoreProvider].getName
+  )
+
+  TEST_PROVIDERS.foreach { provider =>
+    test("SPARK-53103: non empty state and commits checkpoint directory on first batch"
+      + s"(with $provider)") {
+      withSQLConf(
+        SQLConf.STATE_STORE_PROVIDER_CLASS.key -> provider) {
+
+        withTempDir { checkpointDir =>
+          val q = MemoryStream[Int].toDS().groupBy().count()
+            .writeStream
+            .format("memory")
+            .outputMode("complete")
+            .queryName(s"name${RandomStringUtils.secure.nextAlphabetic(10)}")
+            .option("checkpointLocation", checkpointDir.getCanonicalPath)
+            .start()
+          // Verify that the query can start successfully when the checkpoint directory is empty.
+          q.stop()
+        }
+
+        withTempDir { checkpointDir =>
+          val hadoopConf = spark.sessionState.newHadoopConf()
+          val fm = CheckpointFileManager.create(new Path(checkpointDir.toString), hadoopConf)
+
+          // Create a non-empty state checkpoint directory to simulate the case that the user
+          // a directory that already has state data.
+          fm.mkdirs(new Path(new Path(checkpointDir.getCanonicalPath, "state"), "0"))
+
+          checkError(
+            exception = intercept[StreamingQueryException] {
+              MemoryStream[Int].toDS().groupBy().count()
+                .writeStream
+                .format("memory")
+                .outputMode("complete")
+                .queryName(s"name${RandomStringUtils.secure.nextAlphabetic(10)}")
+                .option("checkpointLocation", checkpointDir.getCanonicalPath)
+                .start()
+                .processAllAvailable()
+            }.getCause.asInstanceOf[StateStoreCheckpointLocationNotEmpty],
+            condition = "STATE_STORE_CHECKPOINT_LOCATION_NOT_EMPTY",
+            sqlState = "42K03",
+            parameters = Map(
+              "checkpointLocation" ->
+                ("file:" + (new Path(checkpointDir.getCanonicalPath, "state")).toString)
+            ))
+        }
+
+        withTempDir { checkpointDir =>
+          val hadoopConf = spark.sessionState.newHadoopConf()
+          val fm = CheckpointFileManager.create(new Path(checkpointDir.toString), hadoopConf)
+
+          // Create a non-empty state checkpoint directory to simulate the case that the user
+          // a directory that already has commits data.
+          fm.mkdirs(new Path(new Path(checkpointDir.getCanonicalPath, "commits"), "0"))
+
+          checkError(
+            exception = intercept[StreamingQueryException] {
+              MemoryStream[Int].toDS().groupBy().count()
+                .writeStream
+                .format("memory")
+                .outputMode("complete")
+                .queryName(s"name${RandomStringUtils.secure.nextAlphabetic(10)}")
+                .option("checkpointLocation", checkpointDir.getCanonicalPath)
+                .start()
+                .processAllAvailable()
+            }.getCause.asInstanceOf[StateStoreCheckpointLocationNotEmpty],
+            condition = "STATE_STORE_CHECKPOINT_LOCATION_NOT_EMPTY",
+            sqlState = "42K03",
+            parameters = Map(
+              "checkpointLocation" ->
+                ("file:" + (new Path(checkpointDir.getCanonicalPath, "commits")).toString)
+            ))
+        }
+      }
+    }
+  }
+
   private def checkAppendOutputModeException(df: DataFrame): Unit = {
     withTempDir { outputDir =>
       withTempDir { checkpointDir =>
@@ -1474,7 +1582,7 @@ class StreamingQuerySuite extends StreamTest with BeforeAndAfter with Logging wi
       override def schema: StructType = triggerDF.schema
       override def getOffset: Option[Offset] = Some(LongOffset(0))
       override def getBatch(start: Option[Offset], end: Offset): DataFrame = {
-        sqlContext.internalCreateDataFrame(
+        sqlContext.sparkSession.internalCreateDataFrame(
           triggerDF.queryExecution.toRdd, triggerDF.schema, isStreaming = true)
       }
       override def stop(): Unit = {}
